#!/bin/sh

#########################
# Start of conf section #
#########################

# Number of iterations
NUM_ITERATIONS="5"

# Command that executes the simulator
RUN_SIMULATION="ns ./lab7_opt.tcl"

# Command to compute/print statistics
GET_SR_STATISTICS="sh ./lab7_sr.sh"
GET_GBN_STATISTICS="sh ./lab7_gbn.sh"

# Command to cleanup trace files
CLEANUP_TRACES="rm -f ./lab7.tr"

# Function to compute the required statistics. The output of
# GET_STATISTICS is supposed to be in the format:
# <Some_Keyphrase> .* <required quantity>
# that is, there is a unique key phrase at the beginning of the line,
# followed by an arbitrary number of strings/numbers, followed by
# whitespace, followed by the required quantity at the end of the line.
# compute_stats takes as a sole argument that keyphrase and computes and
# prints the mean, sample stddev, and 95% confidence interval for the
# required quantities.
stats() {
	echo "Statistics:"
	compute_stats 'Unique packets transmitted';
	compute_stats 'Received packets';
	compute_stats 'Mean number of transm/packet';
	compute_stats 'Mean number of errors'
	compute_stats 'Mean delay (secs)'
	compute_stats 'Throughput (Mbps)'
}

#########################
## End of conf section ##
#########################

LC_ALL="C"; export LC_ALL
PATH=$PATH:/usr/local/bin:/bin

SR=2

if which gseq >/dev/null 2>&1; then
	SEQ=gseq
elif which seq >/dev/null 2>&1; then
	SEQ=seq
else
	echo "$0: Can't find seq.";
fi

if which gsed >/dev/null 2>&1; then
	SED=gsed
elif which sed >/dev/null 2>&1; then
	SED=sed
else
	echo "$0: Can't find sed.";
fi

compute_stats() {
	echo "$1:"

	WHAT=`echo "$1" | ${SED} 's/[)(\/\\]/\\\&/g'`
	MEAN=`gawk "/^${WHAT}/"'{
		num++;
		sum += $NF;
	} END {
		printf "\t\t%f", sum / num;
	}' < ${TMPF}`;
	echo "	Mean =${MEAN}"

	gawk "/^${WHAT}/"'{
		num++;
		sum += ($NF - '"${MEAN}"')^2;
	} END {
		sd = sqrt(sum/(num - 1));
		printf "\tStd. Dev. =\t%f\n\tConf. Int. =\t%f\n", sd, (1.96 * sd)/sqrt(num)
	}' < ${TMPF};
}

make_temp_file() {
	if which mktemp >/dev/null 2>&1; then
		TMPTEMPLATE=".lab7-XXXXXXXXXX";
		TMPF=`mktemp "${TMPTEMPLATE}"`;
		if [ $? -ne 0 ]; then
			TMPF="";
		fi
	fi
	if [ -z "${TMPF}" ]; then
		if [ ! -f "${TMPTEMPLATE}" ]; then
			touch "${TMPTEMPLATE}";
			TMPF="${TMPTEMPLATE}";
		else
			echo "$0: Unable to create temp file.";
			exit 1;
		fi
	fi
}

clean_temp_file() {
	[ -n "${TMPF}" -a -f "${TMPF}" ] && rm "${TMPF}"
}

run_simulations() {
	make_temp_file;
	for i in `${SEQ} 1 ${NUM_ITERATIONS}`; do
		echo "Iteration $i/${NUM_ITERATIONS}...";
		${RUN_SIMULATION} $* >/dev/null;
		[ $? -ne 0 ] && exit 1;
		if [ "$SR" -eq 1 ]; then
			${GET_SR_STATISTICS} >> ${TMPF};
		else
			${GET_GBN_STATISTICS} >> ${TMPF};
		fi
		${CLEANUP_TRACES};
	done
}

params=$*;
while [ ! -z "$1" ]; do
	if [ "$1" = "-sr" ]; then
		if [ x"$2" = x1 ]; then
			SR=1;
			echo "Selective-Repeat mode...";
			break;
		elif [ x"$2" = x0 ]; then
			SR=0;
			echo "Go-Back-N mode..."
			break;
		else
			echo "Wrong input arguments";
			exit 1;
		fi
	fi
	shift
done
if [ "$SR" -eq 2 ]; then
	echo "Wrong input params";
	exit 1;
fi

run_simulations "$params"
stats
clean_temp_file;
